@@ -32,7 +32,8 @@ from static_frame.core.util import to_datetime64
 from static_frame.core.util import to_timedelta64
 from static_frame.core.util import YearInitializer
 from static_frame.core.util import YearMonthInitializer
-
+from static_frame.core.util import NameType
+from static_frame.core.util import NAME_DEFAULT
 
 if tp.TYPE_CHECKING:
     import pandas  #pylint: disable = W0611 #pragma: no cover
@@ -55,7 +56,7 @@ class IndexDatetime(Index):
     def __init__(self,
             labels: IndexInitializer,
             *,
-            name: tp.Optional[tp.Hashable] = None
+            name: NameType = NAME_DEFAULT,
             ):
         '''Initializer.
 
