@@ -3,6 +3,8 @@
 import typing as tp
 from collections import deque
 from itertools import zip_longest
+from itertools import repeat
+from copy import deepcopy
 
 import numpy as np
 
@@ -30,11 +32,9 @@ from static_frame.core.util import KEY_ITERABLE_TYPES
 from static_frame.core.util import KEY_MULTIPLE_TYPES
 from static_frame.core.util import resolve_dtype_iter
 from static_frame.core.util import EMPTY_TUPLE
+# from static_frame.core.exception import LocInvalid
 
 
-# if tp.TYPE_CHECKING:
-#     from static_frame.core.type_blocks import TypeBlocks #pylint: disable=W0611 #pragma: no cover
-
 INDEX_LEVEL_SLOTS = (
             'index',
             'targets',
@@ -43,8 +43,6 @@ INDEX_LEVEL_SLOTS = (
             '_length',
             )
 
-
-
 class IndexLevel:
     '''
     A nestable representation of an Index, where labels in that index optionally point to other Index objects.
@@ -174,6 +172,19 @@ class IndexLevel:
             self._depth = depth_reference
             self._length = 0
 
+    #---------------------------------------------------------------------------
+    def __deepcopy__(self, memo: tp.Dict[int, tp.Any]) -> 'IndexLevel':
+        obj = self.__new__(self.__class__)
+        obj.index = deepcopy(self.index, memo)
+        obj.targets = deepcopy(self.targets, memo) # ArrayGO implements __deepcopy__
+        obj.offset = self.offset
+        obj._depth = self._depth
+        obj._length = self._length
+
+        memo[id(self)] = obj
+        return obj #type: ignore
+
+
     #---------------------------------------------------------------------------
     def depths(self) -> tp.Iterator[int]:
         '''
@@ -261,7 +272,6 @@ class IndexLevel:
                         zip_longest(index, targets[1:], fillvalue=None)
                         ):
                     if level_next is not None:
-                        # print(label, level_next.offset, transversed)
                         # if the next offset is zero, we are moving to a component that is under a fresh hierarchy
                         if level_next.offset > 0:
                             delta = level_next.offset - transversed
@@ -271,7 +281,7 @@ class IndexLevel:
                         # get only the incremental addition for this label
                         transversed += delta
                     else:
-                        # we cannot use offset; must to more expensive length of component Levels
+                        # we cannot use offset; get length of component Levels
                         yield label, len(targets[i])
 
         levels = deque(((self, 0),))
@@ -282,14 +292,50 @@ class IndexLevel:
                 continue # do not need to descend
             if level.targets is not None: # terminus
                 next_depth = depth + 1
-                levels.extend([(lvl, next_depth) for lvl in level.targets])
-
+                levels.extend((lvl, next_depth) for lvl in level.targets)
 
     def labels_at_depth(self,
+            depth_level: int = 0
+            ) -> tp.Iterator[tp.Hashable]:
+        '''
+        Generator all labels found at a specified level, realized at full length.
+        '''
+        def get_labels(index: Index,
+                targets: tp.Optional[ArrayGO]
+                ) -> tp.Iterator[tp.Hashable]:
+            if targets is None:
+                yield from index
+            else: # observe the offsets of the next
+                transversed = 0
+                for i, (label, level_next) in enumerate(
+                        zip_longest(index, targets[1:], fillvalue=None)
+                        ):
+                    if level_next is not None:
+                        if level_next.offset > 0:
+                            delta = level_next.offset - transversed
+                        else:
+                            delta = len(targets[i])
+                        yield from repeat(label, delta)
+                        transversed += delta
+                    else:
+                        yield from repeat(label, len(targets[i]))
+
+        levels = deque(((self, 0),))
+        while levels:
+            level, depth = levels.popleft()
+            if depth == depth_level:
+                yield from get_labels(level.index, level.targets)
+                continue # do not need to descend
+            if level.targets is not None: # terminus
+                next_depth = depth + 1
+                levels.extend((lvl, next_depth) for lvl in level.targets)
+
+
+    def index_array_at_depth(self,
             depth_level: int = 0
             ) -> tp.Iterator[np.ndarray]:
         '''
-        Generator of arrays found at a depth level.
+        Generator of arrays found at a depth level. Used in values_at_depth. This does not provide full-width representation.
         '''
         levels = deque(((self, 0),))
         while levels:
@@ -302,7 +348,7 @@ class IndexLevel:
                 levels.extend([(lvl, next_depth) for lvl in level.targets])
 
 
-    def label_nodes_at_depth(self, depth_level: int) -> tp.Iterator[tp.Hashable]:
+    def index_at_depth(self, depth_level: int) -> tp.Iterator[tp.Hashable]:
         '''Given a depth position, iterate over label nodes at that depth. Only nodes will be provided, which for outer depths may not be of length equal to the entire index.
         '''
         if depth_level == 0:
@@ -398,21 +444,21 @@ class IndexLevel:
         return False
 
     def leaf_loc_to_iloc(self,
-            key: tp.Union[tp.Iterable[tp.Hashable], tp.Type[ILoc], tp.Type[HLoc]]
+            key: tp.Union[tp.Iterable[tp.Hashable], ILoc, HLoc]
             ) -> int:
         '''Given an iterable of single-element level keys (a leaf loc), return the iloc value.
 
         Note that key components (level selectors) cannot be slices, lists, or np.ndarray.
         '''
         if isinstance(key, ILoc):
-            return key.key
+            return key.key #type: ignore [return-value]
 
         node = self
         pos = 0
         key_depth_max = len(key) - 1 #type: ignore
 
         # NOTE: rather than a for/enumerate, this could use a while loop on an iter() and explicitly look at next() results to determine if the key matches
-        for key_depth, k in enumerate(key): #type: ignore
+        for key_depth, k in enumerate(key):
             if isinstance(k, KEY_MULTIPLE_TYPES):
                 raise RuntimeError(f'slices cannot be used in a leaf selection into an IndexHierarchy; try HLoc[{key}].')
             if node.targets is not None:
@@ -494,8 +540,8 @@ class IndexLevel:
                     if isinstance(level_targets, IndexLevel):
                         levels.append((level_targets, next_depth, next_offset))
                     else:
-                        levels.extend([(lvl, next_depth, next_offset)
-                                for lvl in level_targets])
+                        levels.extend((lvl, next_depth, next_offset)
+                                for lvl in level_targets)
 
         iloc_count = len(ilocs)
         if iloc_count == 0:
@@ -520,7 +566,7 @@ class IndexLevel:
     @property
     def values(self) -> np.ndarray:
         '''
-        Return an immutable NumPy 2D array of all labels found in this IndexLevels instance. This may coerce types.
+        Return a single immutable NumPy 2D array of all labels found in this IndexLevels instance. This may coerce types.
         '''
         depth_count = self.depth
         shape = self.__len__(), depth_count
@@ -530,6 +576,7 @@ class IndexLevel:
         labels = np.empty(shape, dtype=dtype)
 
         row_count = 0
+        levels: tp.Deque[tp.Tuple['IndexLevel', int, tp.Optional[np.ndarray]]]
         levels = deque(((self, 0, None),)) # order matters
 
         while levels:
@@ -557,8 +604,9 @@ class IndexLevel:
         return labels
 
     def __iter__(self) -> tp.Iterator[tp.Tuple[tp.Hashable, ...]]:
-        # NOTE: this implementation shown to be faster than a recursive purely recursive implementation.
+        # NOTE: this implementation shown to be faster than a purely recursive implementation.
         depth_count = self.depth
+        levels: tp.Deque[tp.Tuple['IndexLevel', int, tp.Optional[np.ndarray]]]
         levels = deque(((self, 0, None),)) # order matters
 
         while levels:
@@ -577,7 +625,7 @@ class IndexLevel:
                     else:
                         row = row_previous.copy()
                     row[depth] = label
-                    levels.append((level_target, depth_next, row)) #type: ignore
+                    levels.append((level_target, depth_next, row))
 
 
     def values_at_depth(self,
@@ -590,7 +638,7 @@ class IndexLevel:
         dtype = tuple(self.dtype_per_depth())[depth_level]
 
         length = self.__len__()
-        # pre allocate array to ensure we use a resovled type
+        # pre allocate array to ensure we use a resolved type
         array = np.empty(length, dtype=dtype)
 
         if length == 0:
@@ -600,23 +648,23 @@ class IndexLevel:
         if depth_level == depth_count - 1:
             # at maximal depth, can concat underlying arrays
             np.concatenate(
-                    tuple(self.labels_at_depth(depth_level)),
+                    tuple(self.index_array_at_depth(depth_level)),
                     out=array
                     )
         else:
-            def gen() -> tp.Iterator[np.ndarray]:
-                for value, size in self.label_widths_at_depth(
-                        depth_level=depth_level):
-                    if dtype.kind == 'O' and isinstance(value, tuple):
-                        # this appears to the only way to do this:
-                        part = np.empty(size, dtype=dtype)
-                        for i in range(size):
-                            part[i] = value
-                        yield part
-                    else:
-                        yield np.full(size, value, dtype=dtype)
+            arrays = []
+            for value, size in self.label_widths_at_depth(
+                    depth_level=depth_level):
+                if dtype.kind == 'O' and isinstance(value, tuple):
+                    # this appears to the only way to do this:
+                    part = np.empty(size, dtype=dtype)
+                    for i in range(size):
+                        part[i] = value
+                    arrays.append(part)
+                else:
+                    arrays.append(np.full(size, value, dtype=dtype))
 
-            np.concatenate(tuple(gen()), out=array)
+            np.concatenate(arrays, out=array)
 
             #NOTE: This alternative form produced a unicode error only on Windows up ot NP 1.17 for some tests
             # start = 0
@@ -777,8 +825,11 @@ class IndexLevelGO(IndexLevel):
             raise RuntimeError('found IndexLevel with None as targets')
         if depth != level.depth:
             raise RuntimeError('level for extension does not have necessary levels.')
-        if tuple(self.index_types()) != tuple(level.index_types()):
-            raise RuntimeError('level for extension does not have corresponding types.')
+
+        for type_self, type_other in zip(self.index_types(), level.index_types()):
+            # self type should be a GO; if other is non-GO, or GO, we can accept it
+            if not issubclass(type_self, type_other):
+                raise RuntimeError(f'level for extension does not have corresponding types: {type_self}, {type_other}')
 
         # this will raise for duplicates
         self.index.extend(level.index.values)
@@ -809,6 +860,8 @@ class IndexLevelGO(IndexLevel):
             raise RuntimeError('appending key {} of insufficent depth {}'.format(
                         key, depth_count))
 
+        level_previous: tp.Optional['IndexLevelGO']
+
         if not self.index.__len__():
             # where we have zero length, create new root index and targets from the key alone
             depth_max = depth_count - 1
@@ -863,7 +916,7 @@ class IndexLevelGO(IndexLevel):
                 # if we have targets, must update them
                 if node.targets is not None:
                     assert level_previous is not None
-                    level_previous.offset = node.__len__()
+                    level_previous.offset = node.__len__() #type: ignore [unreachable]
                     node.targets.append(level_previous)
             else: # depth not found is higher up
                 # NOTE: do not need to use index_from_optional_constructor, as no explicit constructor is being supplied, and we can expect that the existing types must be valid
