@@ -5,43 +5,47 @@ from copy import deepcopy
 
 import numpy as np
 
+from static_frame.core.bus import Bus
 from static_frame.core.container import ContainerBase
 from static_frame.core.container_util import axis_window_items
-from static_frame.core.store_client_mixin import StoreClientMixin
-from static_frame.core.frame import Frame
-from static_frame.core.index_base import IndexBase
-from static_frame.core.bus import Bus
-from static_frame.core.util import NameType
-from static_frame.core.store import StoreConfigMapInitializer
-from static_frame.core.doc_str import doc_inject
-from static_frame.core.display_config import DisplayConfig
 from static_frame.core.display import Display
-from static_frame.core.series import Series
+from static_frame.core.display_config import DisplayConfig
+from static_frame.core.doc_str import doc_inject
 from static_frame.core.exception import AxisInvalid
-from static_frame.core.util import NULL_SLICE
-from static_frame.core.util import GetItemKeyType
-from static_frame.core.util import GetItemKeyTypeCompound
-from static_frame.core.node_selector import InterfaceGetItem
-from static_frame.core.index_hierarchy import IndexHierarchy
-from static_frame.core.hloc import HLoc
-from static_frame.core.util import duplicate_filter
-from static_frame.core.util import INT_TYPES
-from static_frame.core.util import AnyCallable
-from static_frame.core.store import Store
-from static_frame.core.node_iter import IterNodeAxis
-from static_frame.core.node_iter import IterNodeType
-from static_frame.core.node_iter import IterNodeConstructorAxis
-from static_frame.core.node_iter import IterNodeWindow
-
 from static_frame.core.exception import ErrorInitQuilt
 from static_frame.core.exception import NotImplementedAxis
-from static_frame.core.util import get_tuple_constructor
+from static_frame.core.frame import Frame
+from static_frame.core.hloc import HLoc
+from static_frame.core.index_base import IndexBase
+from static_frame.core.index_hierarchy import IndexHierarchy
+from static_frame.core.node_iter import IterNodeAxis
+from static_frame.core.node_iter import IterNodeConstructorAxis
+from static_frame.core.node_iter import IterNodeType
+from static_frame.core.node_iter import IterNodeWindow
+from static_frame.core.node_selector import InterfaceGetItem
+from static_frame.core.series import Series
+from static_frame.core.store import Store
+from static_frame.core.store import StoreConfigMapInitializer
+from static_frame.core.store_client_mixin import StoreClientMixin
+from static_frame.core.store_hdf5 import StoreHDF5
+from static_frame.core.store_sqlite import StoreSQLite
+from static_frame.core.store_xlsx import StoreXLSX
+from static_frame.core.store_zip import StoreZipCSV
+from static_frame.core.store_zip import StoreZipParquet
+from static_frame.core.store_zip import StoreZipPickle
+from static_frame.core.store_zip import StoreZipTSV
+from static_frame.core.util import AnyCallable
 from static_frame.core.util import array_deepcopy
+from static_frame.core.util import duplicate_filter
+from static_frame.core.util import get_tuple_constructor
+from static_frame.core.util import GetItemKeyType
+from static_frame.core.util import GetItemKeyTypeCompound
+from static_frame.core.util import INT_TYPES
+from static_frame.core.util import NameType
+from static_frame.core.util import NULL_SLICE
+from static_frame.core.util import PathSpecifier
 
 
-# from static_frame.core.store import StoreConfigMap
-# from static_frame.core.store import StoreConfigMapInitializer
-
 def get_extractor(
         deepcopy_from_bus: bool,
         is_array: bool,
@@ -204,27 +208,251 @@ class Quilt(ContainerBase, StoreClientMixin):
                 deepcopy_from_bus=deepcopy_from_bus,
                 )
 
+    #---------------------------------------------------------------------------
+    # constructors by data format
+
     @classmethod
     def _from_store(cls,
             store: Store,
             *,
             config: StoreConfigMapInitializer = None,
-            **kwargs: tp.Any,
+            axis: int = 0,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            max_persist: tp.Optional[int] = None,
             ) -> 'Quilt':
-        '''
-        For compatibility with StoreClientMixin.
-        '''
         bus = Bus._from_store(store=store,
                 config=config,
-                max_persist=kwargs.get('max_persist'), # None is default
+                max_persist=max_persist, # None is default
                 )
         return cls(bus,
-                axis=kwargs.get('axis', 0),
-                retain_labels=kwargs['retain_labels'],
-                deepcopy_from_bus=kwargs.get('deepcopy_from_bus', False),
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                )
+
+
+    @classmethod
+    @doc_inject(selector='quilt_constructor')
+    def from_zip_tsv(cls,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            axis: int = 0,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            max_persist: tp.Optional[int] = None,
+            ) -> 'Quilt':
+        '''
+        Given a file path to zipped TSV :obj:`Quilt` store, return a :obj:`Quilt` instance.
+
+        {args}
+        '''
+        store = StoreZipTSV(fp)
+        return cls._from_store(store,
+                config=config,
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                max_persist=max_persist,
+                )
+
+    @classmethod
+    @doc_inject(selector='quilt_constructor')
+    def from_zip_csv(cls,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            axis: int = 0,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            max_persist: tp.Optional[int] = None,
+            ) -> 'Quilt':
+        '''
+        Given a file path to zipped CSV :obj:`Quilt` store, return a :obj:`Quilt` instance.
+
+        {args}
+        '''
+        store = StoreZipCSV(fp)
+        return cls._from_store(store,
+                config=config,
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                max_persist=max_persist,
+                )
+
+    @classmethod
+    @doc_inject(selector='quilt_constructor')
+    def from_zip_pickle(cls,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            axis: int = 0,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            max_persist: tp.Optional[int] = None,
+            ) -> 'Quilt':
+        '''
+        Given a file path to zipped pickle :obj:`Quilt` store, return a :obj:`Quilt` instance.
+
+        {args}
+        '''
+        store = StoreZipPickle(fp)
+        return cls._from_store(store,
+                config=config,
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                max_persist=max_persist,
+                )
+
+
+    @classmethod
+    @doc_inject(selector='quilt_constructor')
+    def from_zip_parquet(cls,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            axis: int = 0,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            max_persist: tp.Optional[int] = None,
+            ) -> 'Quilt':
+        '''
+        Given a file path to zipped parquet :obj:`Quilt` store, return a :obj:`Quilt` instance.
+
+        {args}
+        '''
+        store = StoreZipParquet(fp)
+        return cls._from_store(store,
+                config=config,
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                max_persist=max_persist,
+                )
+
+
+    @classmethod
+    @doc_inject(selector='quilt_constructor')
+    def from_xlsx(cls,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            axis: int = 0,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            max_persist: tp.Optional[int] = None,
+            ) -> 'Quilt':
+        '''
+        Given a file path to an XLSX :obj:`Quilt` store, return a :obj:`Quilt` instance.
+
+        {args}
+        '''
+        # how to pass configuration for multiple sheets?
+        store = StoreXLSX(fp)
+        return cls._from_store(store,
+                config=config,
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                max_persist=max_persist,
+                )
+
+
+    @classmethod
+    @doc_inject(selector='quilt_constructor')
+    def from_sqlite(cls,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            axis: int = 0,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            max_persist: tp.Optional[int] = None,
+            ) -> 'Quilt':
+        '''
+        Given a file path to an SQLite :obj:`Quilt` store, return a :obj:`Quilt` instance.
+
+        {args}
+        '''
+        store = StoreSQLite(fp)
+        return cls._from_store(store,
+                config=config,
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                max_persist=max_persist,
+                )
+
+
+    @classmethod
+    @doc_inject(selector='quilt_constructor')
+    def from_hdf5(cls,
+            fp: PathSpecifier,
+            *,
+            config: StoreConfigMapInitializer = None,
+            axis: int = 0,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            max_persist: tp.Optional[int] = None,
+            ) -> 'Quilt':
+        '''
+        Given a file path to a HDF5 :obj:`Quilt` store, return a :obj:`Quilt` instance.
+
+        {args}
+        '''
+        store = StoreHDF5(fp)
+        return cls._from_store(store,
+                config=config,
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                max_persist=max_persist,
                 )
 
     #---------------------------------------------------------------------------
+
+    @classmethod
+    def from_items(cls,
+            items: tp.Iterable[tp.Tuple[tp.Hashable, Frame]],
+            *,
+            axis: int = 0,
+            name: NameType = None,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            ) -> 'Quilt':
+        '''
+        Given an iterable of pairs of label, :obj:`Frame`, create a :obj:`Quilt`.
+        '''
+        bus = Bus.from_items(items, name=name)
+        return cls(bus,
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                )
+
+    @classmethod
+    def from_frames(cls,
+            frames: tp.Iterable[Frame],
+            *,
+            axis: int = 0,
+            name: NameType = None,
+            retain_labels: bool,
+            deepcopy_from_bus: bool = False,
+            ) -> 'Quilt':
+        '''Return a :obj:`Quilt` from an iterable of :obj:`Frame`; labels will be drawn from :obj:`Frame.name`.
+        '''
+        bus = Bus.from_frames(frames, name=name)
+        return cls(bus,
+                axis=axis,
+                retain_labels=retain_labels,
+                deepcopy_from_bus=deepcopy_from_bus,
+                )
+
+    #---------------------------------------------------------------------------
+    @doc_inject(selector='quilt_init')
     def __init__(self,
             bus: Bus,
             *,
@@ -234,6 +462,9 @@ class Quilt(ContainerBase, StoreClientMixin):
             axis_opposite: tp.Optional[IndexBase] = None,
             deepcopy_from_bus: bool = False,
             ) -> None:
+        '''
+        {args}
+        '''
         self._bus = bus
         self._axis = axis
         self._retain_labels = retain_labels
@@ -283,7 +514,7 @@ class Quilt(ContainerBase, StoreClientMixin):
     @doc_inject()
     def name(self) -> NameType:
         '''{}'''
-        return self._bus._name
+        return self._bus._series._name
 
     def rename(self, name: NameType) -> 'Quilt':
         '''
@@ -300,7 +531,7 @@ class Quilt(ContainerBase, StoreClientMixin):
     #---------------------------------------------------------------------------
 
     def __repr__(self) -> str:
-        '''Provide a display of the :obj:`Quilt` that does not exhaust the generator.
+        '''Provide a display of the :obj:`Quilt` that does not realize the entire :obj:`Frame`.
         '''
         if self.name:
             header = f'{self.__class__.__name__}: {self.name}'
@@ -398,7 +629,7 @@ class Quilt(ContainerBase, StoreClientMixin):
     @property
     def status(self) -> Frame:
         '''
-        Return a :obj:`Frame` indicating loaded status, size, bytes, and shape of all loaded :obj:`Frame` in the contained :obj:`Bus`.
+        Return a :obj:`Frame` indicating loaded status, size, bytes, and shape of all loaded :obj:`Frame` in the contained :obj:`Quilt`.
         '''
         return self._bus.status
 
@@ -452,7 +683,7 @@ class Quilt(ContainerBase, StoreClientMixin):
     # compatibility with StoreClientMixin
 
     def _items_store(self) -> tp.Iterator[tp.Tuple[tp.Hashable, Frame]]:
-        '''Iterator of pairs of :obj:`Bus` label and contained :obj:`Frame`.
+        '''Iterator of pairs of :obj:`Quilt` label and contained :obj:`Frame`.
         '''
         yield from self._bus.items()
 
@@ -653,11 +884,11 @@ class Quilt(ContainerBase, StoreClientMixin):
             sel_component = sel[self._axis_map.index.loc_to_iloc(HLoc[key])]
 
             if self._axis == 0:
-                component = self._bus.loc[key]._extract_array(sel_component, opposite_key)
+                component = self._bus.loc[key]._extract_array(sel_component, opposite_key) #type: ignore [attr-defined]
                 if sel_reduces:
                     component = component[0]
             else:
-                component = self._bus.loc[key]._extract_array(opposite_key, sel_component)
+                component = self._bus.loc[key]._extract_array(opposite_key, sel_component) #type: ignore [attr-defined]
                 if sel_reduces:
                     if component.ndim == 1:
                         component = component[0]
@@ -998,7 +1229,7 @@ class Quilt(ContainerBase, StoreClientMixin):
 
     #---------------------------------------------------------------------------
     # transformations resulting in changed dimensionality
-    @doc_inject(selector='head', class_name='Frame')
+    @doc_inject(selector='head', class_name='Quilt')
     def head(self, count: int = 5) -> 'Frame':
         '''{doc}
 
@@ -1007,7 +1238,7 @@ class Quilt(ContainerBase, StoreClientMixin):
         '''
         return self.iloc[:count]
 
-    @doc_inject(selector='tail', class_name='Frame')
+    @doc_inject(selector='tail', class_name='Quilt')
     def tail(self, count: int = 5) -> 'Frame':
         '''{doc}
 
