@@ -5,7 +5,7 @@ import datetime
 import numpy as np
 
 from static_frame.core.container_util import apex_to_name
-from static_frame.core.doc_str import doc_inject
+# from static_frame.core.doc_str import doc_inject
 from static_frame.core.frame import Frame
 from static_frame.core.index import Index
 from static_frame.core.index_base import IndexBase
@@ -21,26 +21,26 @@ from static_frame.core.store_filter import StoreFilter
 from static_frame.core.util import AnyCallable
 from static_frame.core.util import BOOL_TYPES
 from static_frame.core.util import COMPLEX_TYPES
-# from static_frame.core.util import DT64_MONTH
-# from static_frame.core.util import DT64_YEAR
 from static_frame.core.util import DTYPE_BOOL
 from static_frame.core.util import DTYPE_INEXACT_KINDS
 from static_frame.core.util import DTYPE_INT_KINDS
-# from static_frame.core.util import DTYPE_NAT_KINDS
 from static_frame.core.util import DTYPE_OBJECT
 from static_frame.core.util import DTYPE_STR_KINDS
 from static_frame.core.util import NUMERIC_TYPES
+from static_frame.core.util import array1d_to_last_contiguous_to_edge
+from static_frame.core.util import STORE_LABEL_DEFAULT
 
 if tp.TYPE_CHECKING:
     from xlsxwriter.worksheet import Worksheet  # pylint: disable=W0611 #pragma: no cover
     from xlsxwriter.workbook import Workbook  # pylint: disable=W0611 #pragma: no cover
     from xlsxwriter.format import Format  # pylint: disable=W0611 #pragma: no cover
-
+    from xlsxwriter.format import Format  # pylint: disable=W0611 #pragma: no cover
+    # from openpyxl.cell.read_only import ReadOnlyCell # pylint: disable=W0611 #pragma: no cover
+    # from openpyxl.cell.read_only import EmptyCell # pylint: disable=W0611 #pragma: no cover
 
 MAX_XLSX_ROWS = 1048576
 MAX_XLSX_COLUMNS = 16384 #1024 on libre office
 
-
 class FormatDefaults:
 
     @staticmethod
@@ -72,8 +72,6 @@ class FormatDefaults:
         return f
 
 
-
-
 class StoreXLSX(Store):
 
     _EXT: tp.FrozenSet[str] =  frozenset(('.xlsx',))
@@ -131,7 +129,7 @@ class StoreXLSX(Store):
             if isinstance(value, COMPLEX_TYPES):
                 return ws.write_string(row, col, str(value), format_cell)
 
-            if writer_attr == 'write':
+            if writer_attr == 'write': #type: ignore [unreachable]
                 # determine type for each value
                 if isinstance(value, BOOL_TYPES):
                     return ws.write_boolean(row, col, value, format_cell)
@@ -195,7 +193,7 @@ class StoreXLSX(Store):
             if store_filter:
                 columns_values = store_filter.from_type_filter_array(columns_values)
             writer_columns = cls._get_writer(columns_values.dtype, ws)
-            # for labels in acme, do not know type
+            # for labels in apex, do not know type
             writer_names = cls._get_writer(DTYPE_OBJECT, ws)
 
         # write by column
@@ -283,7 +281,7 @@ class StoreXLSX(Store):
 
     @store_coherent_write
     def write(self,
-            items: tp.Iterable[tp.Tuple[tp.Optional[str], Frame]],
+            items: tp.Iterable[tp.Tuple[tp.Hashable, Frame]],
             *,
             config: StoreConfigMapInitializer = None,
             store_filter: tp.Optional[StoreFilter] = STORE_FILTER_DEFAULT
@@ -306,6 +304,11 @@ class StoreXLSX(Store):
 
         for label, frame in items:
             c = config_map[label]
+            if label is STORE_LABEL_DEFAULT:
+                # None is supported by add_worksheet, below
+                label = None
+            else:
+                label = config_map.default.label_encode(label)
 
             # NOTE: this must be called here, as we need the workbook been assigning formats, and we need to get a config per label
             format_columns = FormatDefaults.get_format_or_default(
@@ -337,7 +340,7 @@ class StoreXLSX(Store):
                     format_funcs=(FormatDefaults.label, FormatDefaults.datetime,))
 
 
-            ws = wb.add_worksheet(label)
+            ws = wb.add_worksheet(label) # label can be None
             self._frame_to_worksheet(frame,
                     ws,
                     format_columns=format_columns,
@@ -366,171 +369,204 @@ class StoreXLSX(Store):
                 data_only=True
                 )
 
-    @doc_inject(selector='constructor_frame')
+    # @doc_inject(selector='constructor_frame')
+    @store_coherent_non_write
+    def read_many(self,
+            labels: tp.Iterable[tp.Hashable],
+            *,
+            config: StoreConfigMapInitializer = None,
+            store_filter: tp.Optional[StoreFilter] = STORE_FILTER_DEFAULT,
+            container_type: tp.Type[Frame] = Frame,
+            ) -> tp.Iterator[Frame]:
+
+        config_map = StoreConfigMap.from_initializer(config)
+        wb = self._load_workbook(self._fp)
+
+        for label in labels:
+            c = config_map[label]
+
+            index_depth = c.index_depth
+            index_name_depth_level = c.index_name_depth_level
+            columns_depth = c.columns_depth
+            columns_name_depth_level = c.columns_name_depth_level
+            trim_nadir = c.trim_nadir
+            skip_header = c.skip_header
+            skip_footer = c.skip_footer
+            dtypes = c.dtypes
+            consolidate_blocks = c.consolidate_blocks
+
+            if label is STORE_LABEL_DEFAULT:
+                ws = wb[wb.sheetnames[0]]
+                name = None # do not set to default sheet name
+            else:
+                label_encoded = config_map.default.label_encode(label)
+                ws = wb[label_encoded]
+                name = label # set name to the un-encoded hashable
+
+            if ws.max_column <= 1 or ws.max_row <= 1:
+                # https://openpyxl.readthedocs.io/en/stable/optimized.html
+                # says that some clients might not report correct dimensions
+                ws.calculate_dimension()
+
+            max_column = ws.max_column
+            max_row = ws.max_row
+
+            # adjust for downward shift for skipping header, then reduce for footer; at this value and beyond we stop
+            last_row_count = max_row - skip_header - skip_footer
+
+            index_values: tp.List[tp.Any] = []
+            columns_values: tp.List[tp.Any] = []
+            data = []
+            apex_rows = []
+
+            if trim_nadir:
+                mask = np.full((last_row_count, max_column), False)
+
+            for row_count, row in enumerate(
+                    ws.iter_rows(max_row=max_row), start=-skip_header):
+                if row_count < 0:
+                    continue # due to skip header; preserves comparison to columns_depth
+                if row_count >= last_row_count:
+                    break
+
+                if trim_nadir:
+                    row_data: tp.Sequence[tp.Any] = []
+                    for col_count, cell in enumerate(row):
+                        if store_filter is None:
+                            value = cell.value
+                        else:
+                            value = store_filter.to_type_filter_element(cell.value)
+                        if value is None: # NOTE: only checking None, not np.nan
+                            mask[row_count, col_count] = True
+                        row_data.append(value) # type: ignore
+                    if not row_data:
+                        mask[row_count] = True
+                else:
+                    if store_filter is None:
+                        row_data = tuple(cell.value for cell in row)
+                    else: # only need to filter string values, but probably too expensive to pre-check
+                        row_data = tuple(store_filter.to_type_filter_element(cell.value) for cell in row)
+
+                if row_count <= columns_depth - 1:
+                    apex_rows.append(row_data[:index_depth])
+                    if columns_depth == 1:
+                        columns_values.extend(row_data[index_depth:])
+                    elif columns_depth > 1:
+                        columns_values.append(row_data[index_depth:])
+                    continue
+
+                if index_depth == 0:
+                    data.append(row_data)
+                elif index_depth == 1:
+                    index_values.append(row_data[0])
+                    data.append(row_data[1:])
+                else:
+                    index_values.append(row_data[:index_depth])
+                    data.append(row_data[index_depth:])
+
+            #-----------------------------------------------------------------------
+            # Trim all-empty trailing rows created from style formatting GH#146. As the wb is opened in read-only mode, reverse iterating on the wb is not an option, nor is direct row access by integer
+            if trim_nadir:
+                # NOTE: `mask` is all data, while `data` is post index/columns extraction; this means that if a non-None label is found, the row/column will not be trimmed.
+                row_mask = mask.all(axis=1)
+                row_trim_start = array1d_to_last_contiguous_to_edge(row_mask) - columns_depth
+                if row_trim_start < len(row_mask) - columns_depth:
+                    data = data[:row_trim_start]
+                    if index_depth > 0: # this handles depth 1 and greater
+                        index_values = index_values[:row_trim_start]
+
+                col_mask = mask.all(axis=0)
+                col_trim_start = array1d_to_last_contiguous_to_edge(col_mask) - index_depth
+                if col_trim_start < len(col_mask) - index_depth:
+                    data = (r[:col_trim_start] for r in data) #type: ignore
+                    if columns_depth == 1:
+                        columns_values = columns_values[:col_trim_start]
+                    if columns_depth > 1:
+                        columns_values = (r[:col_trim_start] for r in columns_values) #type: ignore
+
+            #-----------------------------------------------------------------------
+            # continue with Index and Frame creation
+            index_name = None if columns_depth == 0 else apex_to_name(
+                    rows=apex_rows,
+                    depth_level=index_name_depth_level,
+                    axis=0,
+                    axis_depth=index_depth)
+
+            index: tp.Optional[IndexBase] = None
+            own_index = False
+            if index_depth == 1:
+                index = Index(index_values, name=index_name)
+                own_index = True
+            elif index_depth > 1:
+                index = IndexHierarchy.from_labels(
+                        index_values,
+                        continuation_token=None,
+                        name=index_name,
+                        )
+                own_index = True
+
+            columns_name = None if index_depth == 0 else apex_to_name(
+                        rows=apex_rows,
+                        depth_level=columns_name_depth_level,
+                        axis=1,
+                        axis_depth=columns_depth)
+
+            columns: tp.Optional[IndexBase] = None
+            own_columns = False
+            if columns_depth == 1:
+                columns = container_type._COLUMNS_CONSTRUCTOR(
+                        columns_values,
+                        name=columns_name)
+                own_columns = True
+            elif columns_depth > 1:
+                columns = container_type._COLUMNS_HIERARCHY_CONSTRUCTOR.from_labels(
+                        zip(*columns_values),
+                        continuation_token=None,
+                        name=columns_name,
+                        )
+                own_columns = True
+
+            yield container_type.from_records(data,
+                            index=index,
+                            columns=columns,
+                            dtypes=dtypes,
+                            own_index=own_index,
+                            own_columns=own_columns,
+                            name=name,
+                            consolidate_blocks=consolidate_blocks
+                            )
+        wb.close()
+
     @store_coherent_non_write
     def read(self,
-            label: tp.Optional[str] = None,
+            label: tp.Hashable,
             *,
             config: tp.Optional[StoreConfig] = None,
             store_filter: tp.Optional[StoreFilter] = STORE_FILTER_DEFAULT,
             container_type: tp.Type[Frame] = Frame,
             ) -> Frame:
+        '''Read a single Frame, given by `label`, from the Store. Return an instance of `container_type`. This is a convenience method using ``read_many``.
         '''
-        Args:
-            label: Name of sheet to read from XLSX.
-            container_type: Type of container to be returned, either Frame or a Frame subclass
-
-        '''
-        if config is None:
-            config = StoreConfig() # get default
-
-        index_depth = config.index_depth
-        index_name_depth_level = config.index_name_depth_level
-        columns_depth = config.columns_depth
-        columns_name_depth_level = config.columns_name_depth_level
-
-        skip_header = config.skip_header
-        skip_footer = config.skip_footer
-
-        wb = self._load_workbook(self._fp)
-
-        if label is None:
-            ws = wb[wb.sheetnames[0]]
-            name = None # do not set to default sheet name
-        else:
-            ws = wb[label]
-            name = ws.title
-
-        if ws.max_column <= 1 or ws.max_row <= 1:
-            # https://openpyxl.readthedocs.io/en/stable/optimized.html
-            # says that some clients might not report correct dimensions
-            ws.calculate_dimension()
-
-        max_column = ws.max_column
-        max_row = ws.max_row
-
-        # adjust for downward shift for skipping header, then reduce for footer; at this value and beyond we stop
-        last_row_count = max_row - skip_header - skip_footer
-
-        index_values: tp.List[tp.Any] = []
-        columns_values: tp.List[tp.Any] = []
-
-        data = [] # pre-size with None?
-        apex_rows = []
-
-        for row_count, row in enumerate(ws.iter_rows(max_row=max_row), start=-skip_header):
-            if row_count < 0:
-                continue # due to skip header; perserves comparison to columns_depth
-            if row_count >= last_row_count:
-                break
-
-            if store_filter is None:
-                row = tuple(c.value for c in row)
-            else: # only need to filter string values, but probably too expensive to pre-check
-                row = tuple(store_filter.to_type_filter_element(c.value) for c in row)
-
-            if row_count <= columns_depth - 1:
-                apex_rows.append(row[:index_depth])
-                if columns_depth == 1:
-                    columns_values.extend(row[index_depth:])
-                elif columns_depth > 1:
-                    # NOTE: this orientation will need to be rotated
-                    columns_values.append(row[index_depth:])
-                continue
-
-            if index_depth == 0:
-                data.append(row)
-            elif index_depth == 1:
-                index_values.append(row[0])
-                data.append(row[1:])
-            else:
-                index_values.append(row[:index_depth])
-                data.append(row[index_depth:])
-
-        wb.close()
-
-        # Trim all-empty trailing rows created from style formatting GH#146. As the wb is opened in read-only mode, reverse iterating on the wb is not an option, nor is direct row access by integer; alos, evaluating all rows on forward iteration is expensive. Instead, after collecting all the data in a list and closing the wb, reverse iterate and find rows that are all empty.
-        # NOTE: need to handle case where there are valid index values
-
-        empty_token = (None if store_filter is None
-                else store_filter.to_type_filter_element(None))
-
-        for row_count in range(len(data) - 1, -2, -1):
-            if row_count < 0:
-                break
-            if any(c != empty_token for c in data[row_count]): # try to break early with any
-                break
-            if index_depth == 1 and index_values[row_count] != empty_token:
-                break
-            if index_depth > 1 and any(c != empty_token for c in index_values[row_count]):
-                break
-
-        # row_count is set to the first row that has data or index; can be -1
-        empty_row_idx = row_count + 1 # index of all-empty row
-        if empty_row_idx != len(data):
-            # trim data and index_values, if index_depth > 0
-            data = data[:empty_row_idx]
-            if index_depth > 0:
-                index_values = index_values[:empty_row_idx]
-
-        # continue with Index and Frame creation
-        index_name = None if columns_depth == 0 else apex_to_name(
-                rows=apex_rows,
-                depth_level=index_name_depth_level,
-                axis=0,
-                axis_depth=index_depth)
-
-        index: tp.Optional[IndexBase] = None
-        own_index = False
-        if index_depth == 1:
-            index = Index(index_values, name=index_name)
-            own_index = True
-        elif index_depth > 1:
-            index = IndexHierarchy.from_labels(
-                    index_values,
-                    continuation_token=None,
-                    name=index_name,
-                    )
-            own_index = True
-
-        columns_name = None if index_depth == 0 else apex_to_name(
-                    rows=apex_rows,
-                    depth_level=columns_name_depth_level,
-                    axis=1,
-                    axis_depth=columns_depth)
-
-        columns: tp.Optional[IndexBase] = None
-        own_columns = False
-        if columns_depth == 1:
-            columns = container_type._COLUMNS_CONSTRUCTOR(
-                    columns_values,
-                    name=columns_name)
-            own_columns = True
-        elif columns_depth > 1:
-            columns = container_type._COLUMNS_HIERARCHY_CONSTRUCTOR.from_labels(
-                    zip(*columns_values),
-                    continuation_token=None,
-                    name=columns_name,
-                    )
-            own_columns = True
-
-        # NOTE: this might be a Frame or a FrameGO
-        return container_type.from_records(data, #type: ignore
-                        index=index,
-                        columns=columns,
-                        dtypes=config.dtypes,
-                        own_index=own_index,
-                        own_columns=own_columns,
-                        name=name,
-                        consolidate_blocks=config.consolidate_blocks
-                        )
+        return next(self.read_many((label,), #type: ignore
+                config=config,
+                store_filter=store_filter,
+                container_type=container_type,
+                ))
 
     @store_coherent_non_write
-    def labels(self, strip_ext: bool = True) -> tp.Iterator[str]:
+    def labels(self, *,
+            config: StoreConfigMapInitializer = None,
+            strip_ext: bool = True,
+            ) -> tp.Iterator[tp.Hashable]:
+
+        config_map = StoreConfigMap.from_initializer(config)
+
         wb = self._load_workbook(self._fp)
         labels = tuple(wb.sheetnames)
         wb.close()
-        yield from labels
+
+        for label in labels:
+            yield config_map.default.label_decode(label)
 
 
 
